/* cid.c */

/* cid - aka Calleric ID, aka El' Cid
 *	just a simple little proggy to log caller-id
*/

#include "cid.h"

/* loop til we die */
int STOP = FALSE; 

/* incoming caller id flag */
int INC_CID = FALSE;

/* the caller id structure */
caller_id	cid;

/* the main shiznit */
int main (int argc, char *argv[])  {
	int 		fd, result;
	char 		inbuff[SZ_BUFF], outbuff[SZ_BUFF];
	char		*loc, *copy;
	struct termios 	oldtio, newtio;

	(void) signal (SIGINT, getout);	/* exit cleanly on ctrl-c */ 

	fd = open (MODEMDEVICE, O_RDWR | O_NOCTTY);
	if (fd < 0)  {
		perror (MODEMDEVICE); 
		exit (-1);
	}
	
	tcgetattr (fd, &oldtio);  /* save current port settings */
	
	memset (&newtio, 0, sizeof(newtio));
	newtio.c_cflag = BAUDRATE | CRTSCTS | CS8 | CLOCAL | CREAD;
	newtio.c_iflag = IGNPAR;
	newtio.c_oflag = 0;
	
	/* set input mode (non-canonical, no echo,...) */
	newtio.c_lflag = 0;
	
	newtio.c_cc[VTIME] = 20;	/* read after x tenths of a second */
	newtio.c_cc[VMIN] = 0;		/* read after x characters */
	
	tcflush (fd, TCIFLUSH);
	tcsetattr (fd, TCSANOW, &newtio);

	printf ("Initializing modem...\n");
	write (fd, MODEMINIT, strlen (MODEMINIT));

	memset (&outbuff, 0, SZ_BUFF);			/* init the buff */
	while (STOP == FALSE)  {			/* loop for input */
		result = read(fd, inbuff, SZ_BUFF);	/* grab the buff */
		inbuff[result] = 0;			/* mark end of buff */
		strcat (outbuff, inbuff);		/* build our str */
		/* strip the CR/LFs and tag the spots */
		while ((loc = strstr (outbuff, "\r\n")) != NULL)  {
			memcpy (loc, P_TAG, 2);
		}

		loc = strstr (outbuff, P_TAG);		/* search for CR/LF */

		if (loc == outbuff)  {			/* begin of str */
			shiftleft (outbuff, 2);		/* strip tag on begin */
		}

		if (loc != outbuff && loc != NULL)  {	/* at end of str */
			/* malloc -2 for CR/LF and +1 for NULL */
			if (parse (outbuff))  {		/* parse it out */
				printit ();		/* dump output */
				logit (LOG_FILE);	/* log output */
			}
			copy = malloc (strlen (loc) - 1);
			strcpy (copy, loc + 2);		/* dupe it w/o tag */
			memset (outbuff, 0, SZ_BUFF);	/* reset the buff */
			strcpy (outbuff, copy);		/* shove it back in */
		}
	}

	tcsetattr(fd, TCSANOW, &oldtio);  /* set the port back */
	printf ("Exited cleanly.\n");
	return (0);
}

void shiftleft (char *input, int count)  {
	size_t	len;

	len = strlen (input);
	/* shove start of string to the left 2 bytes */
	memmove (input, input + count, len - count);	
	/* and terminate it */
	memcpy (input + len - count, "\0", 1);
}

/* parse it out to get the good stuff */
int parse (const char *input)  {
	int	ret_val = 0;
	char	*dup, *loc;

	/* printf ("Debug: %s\n", input); */

	dup = strdup (input);

	/* Any tags left over will be the end of our string, null em out */
	while ((loc = strstr (dup, P_TAG)) != NULL)  {
		memcpy (loc, "\0\0", 2);
	}

	if (strcmp (dup, "RING") == 0)  {
		printf ("Modem: RING\n");
	}

	if (strcmp (dup, "OK") == 0)  {
		printf ("Modem: OK\n");
	}

	if (strcmp (dup, "ERROR") == 0)  {
		printf ("Modem: ERROR\n");
	}

	/* w00t! caller id info coming in, flip the flag on */
	if (strstr (dup, CID_DATE) != NULL)  {
		memset (&cid, 0, sizeof (caller_id)); /* init struct */
		INC_CID = TRUE;
	}

	/* copy all strings from the end of their flag to null */
	if (INC_CID == TRUE)  {
		if (strstr (dup, CID_DATE) != NULL)  {
			strcpy (cid.date, dup + strlen (CID_DATE));
		}
		if (strstr (dup, CID_TIME) != NULL)  {
			strcpy (cid.time, dup + strlen (CID_TIME));
		}
		if (strstr (dup, CID_NAME) != NULL)  {
			strcpy (cid.name, dup + strlen (CID_NAME));
		}
		if (strstr (dup, CID_NMBR) != NULL)  {
			strcpy (cid.nmbr, dup + strlen (CID_NMBR));
			/* last entry, turn the flag off */
			INC_CID = FALSE;
			ret_val = TRUE;
		}
	}

	free (dup);
	return (ret_val);
}

void printit ()  {
	printf ("--Call Logged--\n");
	printf ("  Date:   %s\n", cid.date);
	printf ("  Time:   %s\n", cid.time);
	printf ("  Name:   %s\n", cid.name);
	printf ("  Number: %s\n\n", cid.nmbr);
}

void logit (char *logfile)  {
	FILE	*file;
	time_t	unix_time;

	unix_time = time (NULL);

	if ((file = fopen (logfile, "a")) == NULL)  {
		printf ("Failed to write to %s.\n", logfile);
	}  else  {
		fprintf (file, "%s,%s,%s,%s,%ld\n",
			cid.date,
			cid.time,
			cid.name,
			cid.nmbr,
			unix_time);
	}

	fclose (file);
}

/* let ya know we're exiting cleanly */
void getout (int sig)  {
	printf ("Caught signal %d, standby while exiting...\n", sig);
	STOP = TRUE;
}
